@@ -417,9 +417,9 @@ about_me_load_photo (GnomeAboutMe *me, EContact *contact)
 
 	photo = e_contact_get (contact, E_CONTACT_PHOTO);
 
-	if (photo) {
+	if (photo && photo->type == E_CONTACT_PHOTO_TYPE_INLINED) {
 		me->have_image = TRUE;
-		e_image_chooser_set_image_data (E_IMAGE_CHOOSER (widget), photo->data, photo->length);
+		e_image_chooser_set_image_data (E_IMAGE_CHOOSER (widget), photo->data.inlined.data, photo->data.inlined.length);
 		e_contact_photo_free (photo);
 	} else {
 		me->have_image = FALSE;
@@ -435,7 +435,7 @@ about_me_update_photo (GnomeAboutMe *me)
 	gchar         *file;
 	FILE	      *fp;
 
-	gchar 	      *data;
+	guchar 	      *data;
 	gsize 	       length;
 
 	dialog = me->dialog;
@@ -449,20 +449,19 @@ about_me_update_photo (GnomeAboutMe *me)
 		float scale;
 		
 		widget = WID ("image-chooser");
-		e_image_chooser_get_image_data (E_IMAGE_CHOOSER (widget), &data, &length);
+		e_image_chooser_get_image_data (E_IMAGE_CHOOSER (widget), (char **) &data, &length);
 
 		/* Before updating the image in EDS scale it to a reasonable size
 		   so that the user doesn't get an application that does not respond
 		   or that takes 100% CPU */
-		gdk_pixbuf_loader_write (loader, (guchar *)data, length, NULL);
+		gdk_pixbuf_loader_write (loader, data, length, NULL);
+		gdk_pixbuf_loader_close (loader, NULL);
 		
 		pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 		
 		if (pixbuf)
-			gdk_pixbuf_ref (pixbuf);
+			g_object_ref (pixbuf);
 			
-		gdk_pixbuf_loader_close (loader, NULL);
-
 		g_object_unref (loader);
 		
 		height = gdk_pixbuf_get_height (pixbuf);
@@ -490,15 +489,16 @@ about_me_update_photo (GnomeAboutMe *me)
 		}
 
 		photo = g_new0 (EContactPhoto, 1);
-		photo->data = data;
-		photo->length = length;
+		photo->type = E_CONTACT_PHOTO_TYPE_INLINED;
+		photo->data.inlined.data = data;
+		photo->data.inlined.length = length;
 		e_contact_set (me->contact, E_CONTACT_PHOTO, photo);
 
 		/* Save the image for GDM */
 		/* FIXME: I would have to read the default used by the gdmgreeter program */
 		file = g_strdup_printf ("%s/.face", g_get_home_dir ());
 		fp = fopen (file, "wb");
-		fwrite (photo->data, 1, photo->length, fp);
+		fwrite (photo->data.inlined.data, 1, photo->data.inlined.length, fp);
 		fclose (fp);
 
 		g_free (file);	
