@@ -10,6 +10,7 @@
 #define THEME_NAME "X-GNOME-Metatheme/Name"
 #define THEME_COMMENT "X-GNOME-Metatheme/Comment"
 #define GTK_THEME_KEY "X-GNOME-Metatheme/GtkTheme"
+#define GTK_COLOR_SCHEME_KEY "X-GNOME-Metatheme/GtkColorScheme"
 #define METACITY_THEME_KEY "X-GNOME-Metatheme/MetacityTheme"
 #define SAWFISH_THEME_KEY "X-GNOME-Metatheme/SawfishTheme"
 #define ICON_THEME_KEY "X-GNOME-Metatheme/IconTheme"
@@ -352,6 +353,16 @@ gnome_theme_read_meta_theme (GnomeVFSURI *meta_theme_uri)
     }
   meta_theme_info->gtk_theme_name = g_strdup (str);
 
+  str = gnome_desktop_item_get_string (meta_theme_ditem, GTK_COLOR_SCHEME_KEY);
+  if (str != NULL)
+  {
+    gchar *a;
+    meta_theme_info->gtk_color_scheme = g_strdup (str);
+    for (a = meta_theme_info->gtk_color_scheme; *a != '\0'; a++)
+      if (*a == ',')
+        *a = '\n';
+  }
+
   str = gnome_desktop_item_get_string (meta_theme_ditem, METACITY_THEME_KEY);
   if (str == NULL)
     {
@@ -425,6 +436,7 @@ read_icon_theme (GnomeVFSURI *icon_theme_uri)
   else 
     {
       gnome_desktop_item_unref (icon_theme_ditem);
+      g_free (icon_theme_file);
       return NULL;
     }
 
@@ -712,22 +724,21 @@ update_common_theme_dir_index (GnomeVFSURI *theme_index_uri,
 	  handle_change_signal (icon_theme?GNOME_THEME_TYPE_ICON:GNOME_THEME_TYPE_METATHEME,
 				theme_info, GNOME_THEME_CHANGE_CREATED, 0);
 	}
-    }
-  else
-    {
-      if (theme_exists)
-	{
+  } else {
+    if (theme_exists) {
 	  gint cmp;
 
 	  if (icon_theme)
 	    cmp = gnome_theme_icon_info_compare (theme_info, old_theme_info);
 	  else
 	    cmp = gnome_theme_meta_info_compare (theme_info, old_theme_info);
-	  if (cmp != 0)
-	    {
-	      g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
-	      add_data_to_hash_by_name (hash_by_name, name, theme_info);
-	      handle_change_signal (icon_theme?GNOME_THEME_TYPE_ICON:GNOME_THEME_TYPE_METATHEME,
+	  if (cmp != 0) {
+	    // Remove old theme
+	    g_hash_table_remove (hash_by_uri, common_theme_dir);
+   	    remove_data_from_hash_by_name (hash_by_name, ((GnomeThemeIconInfo *)old_theme_info)->name, old_theme_info);
+	    g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
+	    add_data_to_hash_by_name (hash_by_name, name, theme_info);
+	    handle_change_signal (icon_theme?GNOME_THEME_TYPE_ICON:GNOME_THEME_TYPE_METATHEME,
 				    theme_info, GNOME_THEME_CHANGE_CHANGED, 0);
 	      if (icon_theme)
 		gnome_theme_icon_info_free (old_theme_info);
@@ -749,8 +760,6 @@ update_common_theme_dir_index (GnomeVFSURI *theme_index_uri,
 	  else
 	    name = ((GnomeThemeMetaInfo *)old_theme_info)->name;
 
-	  g_hash_table_remove (hash_by_uri, common_theme_dir);
-	  remove_data_from_hash_by_name (hash_by_name, name, old_theme_info);
 	  handle_change_signal (icon_theme?GNOME_THEME_TYPE_ICON:GNOME_THEME_TYPE_METATHEME,
 				old_theme_info, GNOME_THEME_CHANGE_DELETED, 0);
 	  if (icon_theme)
@@ -1481,6 +1490,7 @@ gnome_theme_meta_info_free (GnomeThemeMetaInfo *meta_theme_info)
   g_free (meta_theme_info->monospace_font);
   g_free (meta_theme_info->background_image);
   g_free (meta_theme_info->gtk_theme_name);
+  g_free (meta_theme_info->gtk_color_scheme);
   g_free (meta_theme_info->icon_theme_name);
   g_free (meta_theme_info->metacity_theme_name);
   
@@ -1496,6 +1506,7 @@ gnome_theme_meta_info_print (GnomeThemeMetaInfo *meta_theme_info)
   g_print ("comment: %s\n", meta_theme_info->comment);
   g_print ("icon_file: %s\n", meta_theme_info->icon_file);
   g_print ("gtk_theme_name: %s\n", meta_theme_info->gtk_theme_name);
+  g_print ("gtk_color_scheme: %s\n", meta_theme_info->gtk_color_scheme);
   g_print ("metacity_theme_name: %s\n", meta_theme_info->metacity_theme_name);
   g_print ("icon_theme_name: %s\n", meta_theme_info->icon_theme_name);
   g_print ("sawfish_theme_name: %s\n", meta_theme_info->sawfish_theme_name);
@@ -1571,6 +1582,9 @@ gnome_theme_meta_info_compare (GnomeThemeMetaInfo *a,
   cmp = safe_strcmp (a->gtk_theme_name, b->gtk_theme_name);
   if (cmp != 0) return cmp;
 
+  cmp = safe_strcmp (a->gtk_color_scheme, b->gtk_color_scheme);
+  if (cmp != 0) return cmp;
+
   cmp = safe_strcmp (a->metacity_theme_name, b->metacity_theme_name);
   if (cmp != 0) return cmp;
 
