@@ -130,6 +130,7 @@ create_image (ThemeThumbnailData *theme_thumbnail_data,
   g_object_set (G_OBJECT (settings),
 		"gtk-theme-name", (char *) theme_thumbnail_data->control_theme_name->data,
 		"gtk-font-name", (char *) theme_thumbnail_data->application_font->data,
+		"gtk-icon-theme-name", (char *) theme_thumbnail_data->icon_theme_name->data,
 		NULL);
   theme = meta_theme_load ((char *) theme_thumbnail_data->wm_theme_name->data, NULL);
 
@@ -490,7 +491,8 @@ generate_theme_thumbnail (GnomeThemeMetaInfo *meta_theme_info,
   gint i, rowstride;
   char *pixels;
 
-  g_return_val_if_fail (async_data.set == FALSE, NULL);
+  if (async_data.set == TRUE)
+	  return NULL;
 
   pixbuf = g_hash_table_lookup (theme_hash, meta_theme_info->name);
   if (pixbuf != NULL)
