@@ -15,6 +15,7 @@
 #include "gnome-theme-details.h"
 #include "gnome-theme-installer.h"
 #include "gnome-theme-info.h"
+#include "gtkrc-utils.h"
 
 #define MAX_ELEMENTS_BEFORE_SCROLLING 12
 
@@ -48,6 +49,7 @@ static void load_theme_names                (GtkTreeView        *tree_view,
 static char *path_to_theme_id               (const char *path);
 static void update_color_buttons_from_string (gchar *color_scheme);
 
+void revert_color_scheme_key (GtkWidget *checkbutton, gpointer *data);
 
 static char *
 path_to_theme_id (const char *path)
@@ -135,8 +137,8 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
   gchar *theme_selected;
   GtkTreeModel *model;
   GtkTreeIter iter;
-  GList *theme_list=NULL, *string_list, *list;
-  gchar *theme_dir = NULL;
+  GList *theme_list=NULL, *list;
+  gchar *theme_base=NULL;
   GnomeVFSResult vfs_result;
   GnomeVFSFileInfo *vfs_info;
 
@@ -150,12 +152,12 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
   if (theme_selected != NULL) 
   {
      switch (theme_type) {
-		case THEME_GTK: theme_dir = g_strdup("/gtk-2.0/");
+		case THEME_GTK: theme_base = g_strdup("/gtk-2.0/");
 				theme_list = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2);
 				break;
 		case THEME_ICON: theme_list = gnome_theme_icon_info_find_all();
 				 break;
-		case THEME_WINDOW: theme_dir = g_strdup("/metacity-1/");
+		case THEME_WINDOW: theme_base = g_strdup("/metacity-1/");
 				    theme_list = gnome_theme_info_find_by_type (GNOME_THEME_METACITY);
 				    break;
 		default: theme_list = NULL;
@@ -163,22 +165,25 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
     }
   }
 		
-  string_list = NULL;
+  vfs_info = gnome_vfs_file_info_new ();
 
   for (list = theme_list; list; list = list->next)
   {
     GnomeThemeInfo *info = list->data;
+	gchar *theme_dir = NULL;
+
     if (!strcmp(info->name, theme_selected))
     {
 	if (theme_type == THEME_ICON) 
 		theme_dir = g_strdup(info->path);
 	else 
-		theme_dir = g_strdup_printf("%s/%s", info->path, theme_dir);
+		theme_dir = g_strdup_printf("%s/%s", info->path, theme_base);
 			
-	vfs_info = gnome_vfs_file_info_new ();
 	vfs_result = gnome_vfs_get_file_info (theme_dir,
 					      vfs_info,
 				              GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS);
+	g_free (theme_dir);
+
         if (vfs_result == GNOME_VFS_OK) 
 	{
 		if ((vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
@@ -189,8 +194,50 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
 	} else {
 		gtk_widget_set_sensitive(remove_button, FALSE);
 	}
+
       }
     }
+    gnome_vfs_file_info_unref(vfs_info);
+    g_free (theme_base);
+    g_free (theme_selected);
+}
+
+static void
+update_color_scheme_tab ()
+{
+	GSList *symbolic_colors = NULL;
+	GSList *engines = NULL;
+	gboolean fg, bg, base, text, fg_s, bg_s, enable_colors;
+	gchar *filename, *theme_name;
+	GtkSettings *settings;
+	GladeXML *dialog;
+
+	dialog = gnome_theme_manager_get_theme_dialog ();
+
+	settings = gtk_settings_get_default ();
+	g_object_get (G_OBJECT (settings), "gtk-theme-name", &theme_name, NULL);
+	filename = gtkrc_find_named (theme_name);
+
+	gtkrc_get_details (filename, &engines, &symbolic_colors);
+	fg = (g_slist_find_custom (symbolic_colors, "fg_color", g_str_equal) != NULL);
+	bg = (g_slist_find_custom (symbolic_colors, "bg_color", g_str_equal) != NULL);
+	base = (g_slist_find_custom (symbolic_colors, "base_color", g_str_equal) != NULL);
+	text = (g_slist_find_custom (symbolic_colors, "text_color", g_str_equal) != NULL);
+	fg_s = (g_slist_find_custom (symbolic_colors, "selected_fg_color", g_str_equal) != NULL);
+	bg_s = (g_slist_find_custom (symbolic_colors, "selected_bg_color", g_str_equal) != NULL);
+
+	enable_colors = (fg && bg && base && text && fg_s && bg_s);
+
+	gtk_widget_set_sensitive (WID ("color_scheme_table"), enable_colors);
+	gtk_widget_set_sensitive (WID ("color_scheme_revert_button"), enable_colors);
+
+	if (enable_colors)
+		gtk_widget_hide (WID ("color_scheme_message_hbox"));
+	else
+		gtk_widget_show (WID ("color_scheme_message_hbox"));
+
+	g_free (filename);
+	g_free (theme_name);
 }
 
 static void
@@ -198,11 +245,13 @@ gtk_theme_selection_changed (GtkTreeSelection *selection,
 			     gpointer          data)
 {
   GladeXML *dialog;
-		
+
   dialog = gnome_theme_manager_get_theme_dialog ();
-  
+
   update_gconf_key_from_selection (selection, GTK_THEME_KEY);
   gtk_theme_update_remove_button(selection, WID("control_remove_button"), THEME_GTK);
+
+  update_color_scheme_tab ();
 }
 
 static void
@@ -453,7 +502,7 @@ remove_theme(GtkWidget *button, gpointer data)
 }
 
 void
-color_select (GtkWidget *colorbutton, gpointer dialog)
+color_select (GtkWidget *colorbutton, GladeXML *dialog)
 {
   GConfClient *client = NULL;
   gchar *new_scheme;
@@ -475,12 +524,12 @@ color_select (GtkWidget *colorbutton, gpointer dialog)
   gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[5]);
 
 
-  fg = g_strdup_printf ("fg_color:#%.4x%.4x%.4x\n", colors[0].red, colors[0].green, colors[0].blue);
-  bg = g_strdup_printf ("bg_color:#%.4x%.4x%.4x\n", colors[1].red, colors[1].green, colors[1].blue);
-  text = g_strdup_printf ("text_color:#%.4x%.4x%.4x\n", colors[2].red, colors[2].green, colors[2].blue);
-  base = g_strdup_printf ("base_color:#%.4x%.4x%.4x\n", colors[3].red, colors[3].green, colors[3].blue);
-  selected_fg = g_strdup_printf ("selected_fg_color:#%.4x%.4x%.4x\n", colors[4].red, colors[4].green, colors[4].blue);
-  selected_bg = g_strdup_printf ("selected_bg_color:#%.4x%.4x%.4x", colors[5].red, colors[5].green, colors[5].blue);
+  fg = g_strdup_printf ("fg_color:#%.2x%.2x%.2x\n", colors[0].red, colors[0].green, colors[0].blue);
+  bg = g_strdup_printf ("bg_color:#%.2x%.2x%.2x\n", colors[1].red, colors[1].green, colors[1].blue);
+  text = g_strdup_printf ("text_color:#%.2x%.2x%.2x\n", colors[2].red, colors[2].green, colors[2].blue);
+  base = g_strdup_printf ("base_color:#%.2x%.2x%.2x\n", colors[3].red, colors[3].green, colors[3].blue);
+  selected_fg = g_strdup_printf ("selected_fg_color:#%.2x%.2x%.2x\n", colors[2].red, colors[2].green, colors[2].blue);
+  selected_bg = g_strdup_printf ("selected_bg_color:#%.2x%.2x%.2x", colors[5].red, colors[5].green, colors[5].blue);
 
   new_scheme = g_strconcat (fg, bg, text, base, selected_fg, selected_bg, NULL);
 
@@ -496,34 +545,18 @@ color_select (GtkWidget *colorbutton, gpointer dialog)
 }
 
 void
-toggle_color_scheme_key (GtkWidget *checkbutton, gpointer *data)
+revert_color_scheme_key (GtkWidget *checkbutton, gpointer *data)
 {
   GConfClient *client = NULL;
-  gboolean use_theme_colors;
   GladeXML *dialog;
-  GtkWidget *widget;
-  GtkSettings *settings;
-  gchar *color_scheme = NULL;
 
   dialog = gnome_theme_manager_get_theme_dialog ();
-  use_theme_colors = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
 
-  widget = WID ("color_scheme_table");
-  gtk_widget_set_sensitive (widget, !use_theme_colors);
+  client = gconf_client_get_default ();
+  gconf_client_set_string (client, COLOR_SCHEME_KEY, "", NULL);
+  g_object_unref (G_OBJECT (client));
 
-  if (use_theme_colors)
-  {
-    client = gconf_client_get_default ();
-    gconf_client_set_string (client, COLOR_SCHEME_KEY, "", NULL);
-    g_object_unref (G_OBJECT (client));
-  }
-  else
-  {
-    settings = gtk_settings_get_default ();
-    g_object_get (G_OBJECT (settings), "gtk-color-scheme", &color_scheme, NULL);
-    update_color_buttons_from_string (color_scheme);
-    g_free (color_scheme);
-  }
+  gnome_theme_details_update_from_gconf ();
 }
 
 void
@@ -531,6 +564,7 @@ gnome_theme_details_init (void)
 {
   GtkWidget *parent, *widget;
   GladeXML *dialog;
+  gchar *theme;
 
   if (theme_details_initted)
     return;
@@ -581,14 +615,13 @@ gnome_theme_details_init (void)
   widget = WID ("selected_bg_colorbutton");
   g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
 
-  widget = WID ("use_theme_colors_checkbutton");
-  g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (toggle_color_scheme_key), parent);
-
-  gchar *color_scheme;
-  g_object_get (G_OBJECT (gtk_settings_get_default()), "gtk-color-scheme", &color_scheme, NULL);
-  update_color_buttons_from_string (color_scheme);
+  widget = WID ("color_scheme_revert_button");
+  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (revert_color_scheme_key), parent);
 
+  g_object_get (G_OBJECT (gtk_settings_get_default()), "gtk-color-scheme", &theme, NULL);
+  update_color_buttons_from_string (theme);
 
+  /* general signals */
   g_signal_connect (G_OBJECT (parent), "response", G_CALLBACK (cb_dialog_response), NULL);
   g_signal_connect (G_OBJECT (parent), "delete-event", G_CALLBACK (gtk_true), NULL);
 
@@ -881,6 +914,7 @@ update_color_buttons_from_string (gchar *color_scheme)
   widget = WID ("selected_bg_colorbutton");
   gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[5]);
 
+
 }
 
 void
@@ -888,11 +922,10 @@ gnome_theme_details_update_from_gconf (void)
 {
   GConfClient *client;
   GladeXML *dialog;
-  GtkWidget *tree_view, *widget;
+  GtkWidget *tree_view;
   gchar *theme = NULL;
   GnomeWindowManager *window_manager = NULL;
   GnomeWMSettings wm_settings;
-  gboolean use_theme_colors = TRUE;
 
   gnome_theme_details_init ();
 
@@ -919,23 +952,15 @@ gnome_theme_details_update_from_gconf (void)
   update_list_something (tree_view, theme);
   g_free (theme);
 
-
-  /* color preferences */
-
+  /* update colour scheme tab */
   theme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
-  if (theme)
-  {
-    use_theme_colors = !strcmp ("", theme);
-    update_color_buttons_from_string (theme);
-    g_free (theme);
-  }
-
-  widget = WID ("use_theme_colors_checkbutton");
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), use_theme_colors);
-
-  widget = WID ("color_scheme_table");
-  gtk_widget_set_sensitive (widget, !use_theme_colors);
+  /* if gconf string is not present, we need to look in gtk settings for a value */
+  if (theme == NULL || strcmp (theme, ""))
+    g_object_get (G_OBJECT (gtk_settings_get_default()), "gtk-color-scheme", &theme, NULL);
+  update_color_buttons_from_string (theme);
+  g_free (theme);
 
+  update_color_scheme_tab ();
 
   g_object_unref (client);
 }
